"use strict";
/**
 * Utils - Script Style
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.outputStyles = exports.outputScripts = exports.addStyle = exports.addScript = void 0;
/* Imports */
const config_js_1 = require("../../config/config.js");
const array_js_1 = require("../array/array.js");
const string_js_1 = require("../string/string.js");
/**
 * Add style or script path and dependencies to build map
 *
 * @private
 * @param {string} type - styles | scripts
 * @param {string} path
 * @param {string[]} [deps]
 * @return {boolean}
 */
const _addScriptStyle = (type, path, deps = []) => {
    /* Path required */
    if (!(0, string_js_1.isStringStrict)(path)) {
        return false;
    }
    /* Input and output dir */
    const scripts = config_js_1.config[type];
    const ext = type === 'scripts' ? 'js' : 'scss';
    const { inputDir, outputDir } = scripts;
    const input = `${inputDir}/${path}.${ext}`;
    const output = `${outputDir}/${path}`;
    /* Add to build */
    scripts.build.set(output, input);
    /* Add deps */
    if ((0, array_js_1.isArrayStrict)(deps)) {
        deps.forEach((dep) => {
            const depInput = `${inputDir}/${dep}.${ext}`;
            const depOutput = `${outputDir}/${dep}`;
            if (!scripts.deps.has(output)) {
                scripts.deps.set(output, new Set());
            }
            scripts.deps.get(output)?.add(depOutput);
            scripts.build.set(depOutput, depInput);
        });
    }
    /* Append success */
    return true;
};
/**
 * Add script path and dependencies to build map
 *
 * @param {string} path
 * @param {string[]} [deps]
 * @return {boolean}
 */
const addScript = (path, deps = []) => {
    return _addScriptStyle('scripts', path, deps);
};
exports.addScript = addScript;
/**
 * Add style path and dependencies to build map
 *
 * @param {string} path
 * @param {string[]} [deps]
 * @return {boolean}
 */
const addStyle = (path, deps = []) => {
    return _addScriptStyle('styles', path, deps);
};
exports.addStyle = addStyle;
/**
 * Output script or stylesheet elements
 *
 * @private
 * @param {string} type - styles | scripts
 * @param {string} link
 * @return {string}
 */
const _outputScriptsStyles = (type, link) => {
    /* Link is required */
    if (!(0, string_js_1.isStringStrict)(link)) {
        return '';
    }
    /* Vars */
    const { build, deps } = config_js_1.config[type];
    /* Check if scripts exist */
    if (build.size === 0) {
        return '';
    }
    /* Sort by dependencies */
    const buildOut = Array.from(build.keys());
    buildOut.sort((a, b) => {
        const bIsDep = deps.get(a)?.has(b);
        const aIsDep = deps.get(b)?.has(a);
        if (bIsDep === true) {
            return -1;
        }
        if (aIsDep === true) {
            return 1;
        }
        return 0;
    });
    /* Output */
    let output = '';
    buildOut.forEach((out) => {
        if (type === 'scripts') {
            output += `<script type="module" src="${link}${out}.js"></script>`;
        }
        if (type === 'styles') {
            output += `<link rel="stylesheet" href="${link}${out}.css" media="all">`;
        }
    });
    return output;
};
/**
 * Output script elements
 *
 * @param {string} link
 * @return {string}
 */
const outputScripts = (link) => {
    return _outputScriptsStyles('scripts', link);
};
exports.outputScripts = outputScripts;
/**
 * Output stylesheet elements
 *
 * @param {string} link
 * @return {string}
 */
const outputStyles = (link) => {
    return _outputScriptsStyles('styles', link);
};
exports.outputStyles = outputStyles;
