"use strict";
/**
 * Utils - WordPress Data Normal
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeWordPressData = void 0;
const block_serialization_spec_parser_1 = require("@wordpress/block-serialization-spec-parser");
const string_js_1 = require("../string/string.js");
const array_js_1 = require("../array/array.js");
const object_js_1 = require("../object/object.js");
const config_js_1 = require("../../config/config.js");
/**
 * Convert blocks to flatter props
 *
 * @private
 * @param {Block[]} blocks
 * @return {RenderItem[]}
 */
const _normalizeBlocks = (blocks) => {
    const newItems = [];
    /* Blocks must be array */
    if (!(0, array_js_1.isArrayStrict)(blocks)) {
        return [];
    }
    /* Recurse */
    blocks.forEach((block) => {
        if (!(0, object_js_1.isObjectStrict)(block)) {
            return;
        }
        const { blockName, attrs, innerBlocks } = block;
        const contentType = blockName;
        if (!(0, string_js_1.isStringStrict)(contentType)) {
            return;
        }
        const attributes = (0, object_js_1.isObjectStrict)(attrs) ? attrs : {};
        const newItem = {
            contentType,
            ...attributes
        };
        if ((0, string_js_1.isString)(config_js_1.config.renderTypes[contentType])) {
            newItem.renderType = config_js_1.config.renderTypes[contentType];
        }
        if ((0, array_js_1.isArrayStrict)(innerBlocks)) {
            newItem.content = _normalizeBlocks(innerBlocks);
        }
        return newItems.push(newItem);
    });
    /* Output */
    return newItems;
};
/**
 * Transform item props to flatter structure
 *
 * @private
 * @param {WordPressDataItem} item
 * @return {RenderItem}
 */
const _normalizeItem = (item) => {
    const newItem = {};
    /* Loop item */
    for (const [key, value] of Object.entries(item)) {
        /* Value */
        let val = value;
        /* Content type */
        if (key === 'type' && (0, string_js_1.isStringStrict)(value)) {
            newItem.contentType = value;
            if ((0, string_js_1.isString)(config_js_1.config.renderTypes[value])) {
                newItem.renderType = config_js_1.config.renderTypes[value];
            }
            continue;
        }
        /* Flatten rendered */
        if ((0, object_js_1.isObjectStrict)(value) && (0, string_js_1.isString)(value.rendered)) {
            val = value.rendered;
        }
        /* Excerpt */
        if (key === 'excerpt') {
            val = val.replace(/<(?:"[^"]*"['"]*|'[^']*'['"]*|[^'">])+>/g, '');
        }
        /* Content */
        if (key === 'content' && (0, string_js_1.isString)(val)) {
            val = _normalizeBlocks((0, block_serialization_spec_parser_1.parse)(val));
        }
        /* Set value */
        newItem[key] = val;
    }
    /* Output */
    return newItem;
};
/**
 * Transform wordpress data into simpler objects
 *
 * @param {WordPressDataItem[]} data
 * @param {RenderItem[]} [_newData]
 * @return {RenderItem[]}
 */
const normalizeWordPressData = (data, _newData = []) => {
    if (!(0, array_js_1.isArrayStrict)(data)) {
        return [];
    }
    /* Recurse data */
    data.forEach((item) => {
        if (!(0, object_js_1.isObjectStrict)(item)) {
            return;
        }
        _newData.push(_normalizeItem(item));
    });
    /* Output */
    return _newData;
};
exports.normalizeWordPressData = normalizeWordPressData;
